/*
 The software in this package is distributed under the GNU General
 Public License version 2 (with a special exception described below).

 A copy of GNU General Public License (GPL) is included in this distribution,
 in the file COPYING.GPL.

 As a special exception, if other files instantiate templates or use macros
 or inline functions from this file, or you compile this file and link it
 with other works to produce a work based on this file, this file
 does not by itself cause the resulting work to be covered
 by the GNU General Public License.

 However the source code for this file must still be made available
 in accordance with section (3) of the GNU General Public License.

 This exception does not invalidate any other reasons why a work based
 on this file might be covered by the GNU General Public License.
 */

#include "config/option/hostpingintervaloption.h"

#include <string>

#include <boost/foreach.hpp>
#include <boost/lexical_cast.hpp>

#include <stringfunc.hxx>   // I2n::trim

#include <logfunc.hpp>

#include "boost_assert_handler.h"

using namespace std;
using boost::program_options::value;
using boost::program_options::variables_map;
using I2n::Logger::GlobalLogger;

//-----------------------------------------------------------------------------
// HostPingIntervalOption
//-----------------------------------------------------------------------------

const std::string range_separator = "..";
const int failure_interval = 60;

HostPingIntervalOption::HostPingIntervalOption() :
    HostConfigurationOption(
        "host.interval",
        value< vector<std::string> >(),
        "Interval between each ping to the host."
    )
{
}

HostPingIntervalOption::~HostPingIntervalOption()
{
}

bool HostPingIntervalOption::parse(
        const variables_map& vm,
        Configuration *configuration
)
{
    size_t hosts_count = 0;
    size_t hosts_intervals_count = 0;
    bool parsed_success = false;
    size_t range_separator_pos = 0;
    std::string interval_part;
    int host_interval = 0;
    int host_interval_lower = 0;
    int host_interval_upper = 0;

    // [host] interval
    if ( 1 <= vm.count( get_command_string() ) )
    {
        HostList hosts_list = configuration->get_hosts();
        HostList::iterator hosts_list_iterator = hosts_list.begin();
        hosts_count = hosts_list.size();

        vector<std::string> hosts_intervals_list =
                            vm[ get_command_string() ].as< vector<std::string> >();
        hosts_intervals_count = hosts_intervals_list.size();

        BOOST_ASSERT( hosts_count >= hosts_intervals_count );

        parsed_success = true;

        BOOST_FOREACH( std::string host_interval_str, hosts_intervals_list )
        {
            HostItem host_item = *hosts_list_iterator;

            // look for range_separator
            host_interval_str = I2n::trim(host_interval_str);
            range_separator_pos = host_interval_str.find(range_separator);

            try
            {
                if (range_separator_pos == std::string::npos)
                {
                    // no separator --> assume a fixed interval as single number
                    host_interval = boost::lexical_cast<int>(host_interval_str);
                    BOOST_ASSERT( 0 < host_interval );
                    GlobalLogger.info() << get_command_string() << "="
                            << host_interval << endl;
                }
                else
                {
                    // found separator --> chose interval value from interval
                    interval_part = I2n::trim(host_interval_str.substr(0, range_separator_pos));
                    host_interval_lower = boost::lexical_cast<int>(interval_part);
                    interval_part = I2n::trim(host_interval_str.substr(
                                  range_separator_pos + range_separator.length()));
                    host_interval_upper = boost::lexical_cast<int>(interval_part);

                    BOOST_ASSERT( (0 < host_interval_lower) &&
                            (host_interval_lower <= host_interval_upper) );

                    // create random number within this interval
                    host_interval = configuration->get_random_number(host_interval_lower,
                                                                     host_interval_upper);

                    GlobalLogger.info() << get_command_string() << "=" << host_interval
                        << " from range " << host_interval_lower << ".." << host_interval_upper
                        << endl;
                }
                host_item->set_interval_in_sec( host_interval );
            }

            catch (boost::bad_lexical_cast &exc)
            {
                GlobalLogger.error() << "Error interpreting host ping interval: could not interprete \""
                    << host_interval_str << "\"! Set interval to 60s";
                host_item->set_interval_in_sec( failure_interval );
                parsed_success = false;
            }

            ++hosts_list_iterator;
        } //eo foreach host interval
    }

    set_hosts_count( hosts_intervals_count );

    BOOST_ASSERT( hosts_count == hosts_intervals_count );

    return parsed_success;
}
